@@ -1,20 +1,22 @@
-from probatus.utils import (
-    preprocess_data,
-    shap_calc,
-    calculate_shap_importance,
-    BaseFitComputePlotClass,
-    preprocess_labels,
-    get_single_scorer,
-)
-import numpy as np
-import matplotlib.pyplot as plt
-import pandas as pd
-from sklearn.model_selection._search import BaseSearchCV
-from sklearn.model_selection import check_cv
-from sklearn.base import clone, is_classifier
-from joblib import Parallel, delayed
 import warnings
 
+import matplotlib.pyplot as plt
+import numpy as np
+import pandas as pd
+from joblib import Parallel, delayed
+from probatus.utils import (
+    BaseFitComputePlotClass,
+    assure_pandas_series,
+    calculate_shap_importance,
+    get_single_scorer,
+    preprocess_data,
+    preprocess_labels,
+    shap_calc,
+)
+from sklearn.base import clone, is_classifier
+from sklearn.model_selection import check_cv
+from sklearn.model_selection._search import BaseSearchCV
+
 
 class ShapRFECV(BaseFitComputePlotClass):
     """
@@ -163,7 +165,6 @@ class ShapRFECV(BaseFitComputePlotClass):
                 reproducible results set it to an integer.
         """  # noqa
         self.clf = clf
-
         if isinstance(self.clf, BaseSearchCV):
             self.search_clf = True
         else:
@@ -250,7 +251,9 @@ class ShapRFECV(BaseFitComputePlotClass):
 
     @staticmethod
     def _calculate_number_of_features_to_remove(
-        current_num_of_features, num_features_to_remove, min_num_features_to_keep
+        current_num_of_features,
+        num_features_to_remove,
+        min_num_features_to_keep,
     ):
         """
         Calculates the number of features to be removed.
@@ -332,8 +335,16 @@ class ShapRFECV(BaseFitComputePlotClass):
 
         self.report_df = pd.concat([self.report_df, current_row], axis=0)
 
-    @staticmethod
-    def _get_feature_shap_values_per_fold(X, y, clf, train_index, val_index, scorer, verbose=0, **shap_kwargs):
+    def _get_feature_shap_values_per_fold(
+        self,
+        X,
+        y,
+        clf,
+        train_index,
+        val_index,
+        sample_weight=None,
+        **shap_kwargs,
+    ):
         """
         This function calculates the shap values on validation set, and Train and Val score.
 
@@ -353,17 +364,12 @@ class ShapRFECV(BaseFitComputePlotClass):
             val_index (np.array):
                 Positions of validation fold samples.
 
-            scorer (string, callable or None):
-                A string (see sklearn [model scoring](https://scikit-learn.org/stable/modules/model_evaluation.html)) or
-                a scorer callable object, function with the signature `scorer(estimator, X, y)`.
-
-            verbose (int, optional):
-                Controls verbosity of the output:
-
-                - 0 - neither prints nor warnings are shown
-                - 1 - 50 - only most important warnings regarding data properties are shown (excluding SHAP warnings)
-                - 51 - 100 - shows most important warnings, prints of the feature removal process
-                - above 100 - presents all prints and all warnings (including SHAP warnings).
+            sample_weight (pd.Series, np.ndarray, list, optional):
+                array-like of shape (n_samples,) - only use if the model you're using supports
+                sample weighting (check the corresponding scikit-learn documentation).
+                Array of weights that are assigned to individual samples.
+                Note that they're only used for fitting of  the model, not during evaluation of metrics.
+                If not provided, then each sample is given unit weight.
 
             **shap_kwargs:
                 keyword arguments passed to
@@ -378,18 +384,20 @@ class ShapRFECV(BaseFitComputePlotClass):
         X_train, X_val = X.iloc[train_index, :], X.iloc[val_index, :]
         y_train, y_val = y.iloc[train_index], y.iloc[val_index]
 
-        # Fit model with train folds
-        clf = clf.fit(X_train, y_train)
+        if sample_weight is not None:
+            clf = clf.fit(X_train, y_train, sample_weight=sample_weight.iloc[train_index])
+        else:
+            clf = clf.fit(X_train, y_train)
 
         # Score the model
-        score_train = scorer(clf, X_train, y_train)
-        score_val = scorer(clf, X_val, y_val)
+        score_train = self.scorer.scorer(clf, X_train, y_train)
+        score_val = self.scorer.scorer(clf, X_val, y_val)
 
         # Compute SHAP values
-        shap_values = shap_calc(clf, X_val, verbose=verbose, **shap_kwargs)
+        shap_values = shap_calc(clf, X_val, verbose=self.verbose, **shap_kwargs)
         return shap_values, score_train, score_val
 
-    def fit(self, X, y, columns_to_keep=None, column_names=None, **shap_kwargs):
+    def fit(self, X, y, sample_weight=None, columns_to_keep=None, column_names=None, **shap_kwargs):
         """
         Fits the object with the provided data.
 
@@ -409,6 +417,13 @@ class ShapRFECV(BaseFitComputePlotClass):
             y (pd.Series):
                 Binary labels for X.
 
+            sample_weight (pd.Series, np.ndarray, list, optional):
+                array-like of shape (n_samples,) - only use if the model you're using supports
+                sample weighting (check the corresponding scikit-learn documentation).
+                Array of weights that are assigned to individual samples.
+                Note that they're only used for fitting of  the model, not during evaluation of metrics.
+                If not provided, then each sample is given unit weight.
+
             columns_to_keep (list of str, optional):
                 List of column names to keep. If given,
                 these columns will not be eliminated by the feature elimination process.
@@ -465,6 +480,12 @@ class ShapRFECV(BaseFitComputePlotClass):
 
         self.X, self.column_names = preprocess_data(X, X_name="X", column_names=column_names, verbose=self.verbose)
         self.y = preprocess_labels(y, y_name="y", index=self.X.index, verbose=self.verbose)
+        if sample_weight is not None:
+            if self.verbose > 0:
+                warnings.warn(
+                    "sample_weight is passed only to the fit method of the model, not the evaluation metrics."
+                )
+            sample_weight = assure_pandas_series(sample_weight, index=self.X.index)
         self.cv = check_cv(self.cv, self.y, classifier=is_classifier(self.clf))
 
         remaining_features = current_features_set = self.column_names
@@ -513,8 +534,7 @@ class ShapRFECV(BaseFitComputePlotClass):
                     clf=current_clf,
                     train_index=train_index,
                     val_index=val_index,
-                    scorer=self.scorer.scorer,
-                    verbose=self.verbose,
+                    sample_weight=sample_weight,
                     **shap_kwargs,
                 )
                 for train_index, val_index in self.cv.split(current_X, self.y)
@@ -571,7 +591,7 @@ class ShapRFECV(BaseFitComputePlotClass):
 
         return self.report_df
 
-    def fit_compute(self, X, y, columns_to_keep=None, column_names=None, **shap_kwargs):
+    def fit_compute(self, X, y, sample_weight=None, columns_to_keep=None, column_names=None, **shap_kwargs):
         """
         Fits the object with the provided data.
 
@@ -592,6 +612,13 @@ class ShapRFECV(BaseFitComputePlotClass):
             y (pd.Series):
                 Binary labels for X.
 
+            sample_weight (pd.Series, np.ndarray, list, optional):
+                array-like of shape (n_samples,) - only use if the model you're using supports
+                sample weighting (check the corresponding scikit-learn documentation).
+                Array of weights that are assigned to individual samples.
+                Note that they're only used for fitting of  the model, not during evaluation of metrics.
+                If not provided, then each sample is given unit weight.
+
             columns_to_keep (list of str, optional):
                 List of columns to keep. If given, these columns will not be eliminated.
 
@@ -612,7 +639,14 @@ class ShapRFECV(BaseFitComputePlotClass):
                 DataFrame containing results of feature elimination from each iteration.
         """
 
-        self.fit(X, y, columns_to_keep=columns_to_keep, column_names=column_names, **shap_kwargs)
+        self.fit(
+            X,
+            y,
+            sample_weight=sample_weight,
+            columns_to_keep=columns_to_keep,
+            column_names=column_names,
+            **shap_kwargs,
+        )
         return self.compute()
 
     def get_reduced_features_set(self, num_features):
@@ -695,3 +729,260 @@ class ShapRFECV(BaseFitComputePlotClass):
         else:
             plt.close()
         return ax
+
+
+class EarlyStoppingShapRFECV(ShapRFECV):
+    """
+    This class performs Backwards Recursive Feature Elimination, using SHAP feature importance.
+
+    This is a child of ShapRFECV which allows early stopping of the training step, available in models such as
+        XGBoost and LightGBM. If you are not using early stopping, you should use the parent class,
+        ShapRFECV, instead of EarlyStoppingShapRFECV.
+
+    [Early stopping](https://en.wikipedia.org/wiki/Early_stopping) is a type of
+        regularization technique in which the model is trained until the scoring metric, measured on a validation set,
+        stops improving after a number of early_stopping_rounds. In boosted tree models, this technique can increase
+        the training speed, by skipping the training of trees that do not improve the scoring metric any further,
+        which is particularly useful when the training dataset is large.
+
+    Note that if the classifier is a hyperparameter search model is used, the early stopping parameter is passed only
+        to the fit method of the model duiring the Shapley values estimation step, and not for the hyperparameter
+        search step.
+        Early stopping can be seen as a type of regularization of the optimal number of trees. Therefore you can use
+        it directly with a LightGBM or XGBoost model, as an alternative to a hyperparameter search model.
+
+    At each round, for a
+        given feature set, starting from all available features, the following steps are applied:
+
+    1. (Optional) Tune the hyperparameters of the model using sklearn compatible search CV e.g.
+        [GridSearchCV](https://scikit-learn.org/stable/modules/generated/sklearn.linear_model.LassoCV.html),
+        [RandomizedSearchCV](https://scikit-learn.org/stable/modules/generated/sklearn.model_selection.RandomizedSearchCV.html?highlight=randomized#sklearn.model_selection.RandomizedSearchCV), or
+        [BayesSearchCV](https://scikit-optimize.github.io/stable/modules/generated/skopt.BayesSearchCV.html).
+        Note that during this step the model does not use early stopping.
+    2. Apply Cross-validation (CV) to estimate the SHAP feature importance on the provided dataset. In each CV
+        iteration, the model is fitted on the train folds, and applied on the validation fold to estimate
+        SHAP feature importance. The model is trained until the scoring metric eval_metric, measured on the
+        validation fold, stops improving after a number of early_stopping_rounds.
+    3. Remove `step` lowest SHAP importance features from the dataset.
+
+    At the end of the process, the user can plot the performance of the model for each iteration, and select the
+        optimal number of features and the features set.
+
+    We recommend using [LGBMClassifier](https://lightgbm.readthedocs.io/en/latest/pythonapi/lightgbm.LGBMClassifier.html),
+        because by default it handles missing values and categorical features. In case of other models, make sure to
+        handle these issues for your dataset and consider impact it might have on features importance.
+
+
+    Example:
+    ```python
+    from lightgbm import LGBMClassifier
+    import pandas as pd
+    from probatus.feature_elimination import EarlyStoppingShapRFECV
+    from sklearn.datasets import make_classification
+
+    feature_names = [
+        'f1', 'f2', 'f3', 'f4', 'f5', 'f6', 'f7',
+        'f8', 'f9', 'f10', 'f11', 'f12', 'f13',
+        'f14', 'f15', 'f16', 'f17', 'f18', 'f19', 'f20']
+
+    # Prepare two samples
+    X, y = make_classification(n_samples=200, class_sep=0.05, n_informative=6, n_features=20,
+                               random_state=0, n_redundant=10, n_clusters_per_class=1)
+    X = pd.DataFrame(X, columns=feature_names)
+
+    # Prepare model
+    clf = LGBMClassifier(n_estimators=200, max_depth=3)
+
+    # Run feature elimination
+    shap_elimination = EarlyStoppingShapRFECV(
+        clf=clf, step=0.2, cv=10, scoring='roc_auc', early_stopping_rounds=10, n_jobs=3)
+    report = shap_elimination.fit_compute(X, y)
+
+    # Make plots
+    performance_plot = shap_elimination.plot()
+
+    # Get final feature set
+    final_features_set = shap_elimination.get_reduced_features_set(num_features=3)
+    ```
+    <img src="../img/earlystoppingshaprfecv.png" width="500" />
+
+    """  # noqa
+
+    def __init__(
+        self,
+        clf,
+        step=1,
+        min_features_to_select=1,
+        cv=None,
+        scoring="roc_auc",
+        n_jobs=-1,
+        verbose=0,
+        random_state=None,
+        early_stopping_rounds=5,
+        eval_metric="auc",
+    ):
+        """
+        This method initializes the class.
+
+        Args:
+            clf (binary classifier, sklearn compatible search CV e.g. GridSearchCV, RandomizedSearchCV or BayesSearchCV):
+                A model that will be optimized and trained at each round of features elimination. The model must
+                support early stopping of training, which is the case for XGBoost and LightGBM, for example. The
+                recommended model is [LGBMClassifier](https://lightgbm.readthedocs.io/en/latest/pythonapi/lightgbm.LGBMClassifier.html),
+                because it by default handles the missing values and categorical variables. This parameter also supports
+                any hyperparameter search schema that is consistent with the sklearn API e.g.
+                [GridSearchCV](https://scikit-learn.org/stable/modules/generated/sklearn.model_selection.GridSearchCV.html),
+                [RandomizedSearchCV](https://scikit-learn.org/stable/modules/generated/sklearn.model_selection.RandomizedSearchCV.html)
+                or [BayesSearchCV](https://scikit-optimize.github.io/stable/modules/generated/skopt.BayesSearchCV.html#skopt.BayesSearchCV).
+                Note that if a hyperparemeter search model is used, the hyperparameters are tuned without early
+                stopping. Early stopping is applied only during the Shapley values estimation for feature
+                elimination. We recommend simply passing the model without hyperparameter optimization, or using
+                ShapRFECV without early stopping.
+
+
+            step (int or float, optional):
+                Number of lowest importance features removed each round. If it is an int, then each round such number of
+                features is discarded. If float, such percentage of remaining features (rounded down) is removed each
+                iteration. It is recommended to use float, since it is faster for a large number of features, and slows
+                down and becomes more precise towards less features. Note: the last round may remove fewer features in
+                order to reach min_features_to_select.
+                If columns_to_keep parameter is specified in the fit method, step is the number of features to remove after
+                keeping those columns.
+
+            min_features_to_select (int, optional):
+                Minimum number of features to be kept. This is a stopping criterion of the feature elimination. By
+                default the process stops when one feature is left. If columns_to_keep is specified in the fit method,
+                it may overide this parameter to the maximum between length of columns_to_keep the two.
+
+            cv (int, cross-validation generator or an iterable, optional):
+                Determines the cross-validation splitting strategy. Compatible with sklearn
+                [cv parameter](https://scikit-learn.org/stable/modules/generated/sklearn.feature_selection.RFECV.html).
+                If None, then cv of 5 is used.
+
+            scoring (string or probatus.utils.Scorer, optional):
+                Metric for which the model performance is calculated. It can be either a metric name  aligned with predefined
+                [classification scorers names in sklearn](https://scikit-learn.org/stable/modules/model_evaluation.html).
+                Another option is using probatus.utils.Scorer to define a custom metric.
+
+            n_jobs (int, optional):
+                Number of cores to run in parallel while fitting across folds. None means 1 unless in a
+                `joblib.parallel_backend` context. -1 means using all processors.
+
+            verbose (int, optional):
+                Controls verbosity of the output:
+
+                - 0 - nether prints nor warnings are shown
+                - 1 - 50 - only most important warnings
+                - 51 - 100 - shows other warnings and prints
+                - above 100 - presents all prints and all warnings (including SHAP warnings).
+
+            random_state (int, optional):
+                Random state set at each round of feature elimination. If it is None, the results will not be
+                reproducible and in random search at each iteration a different hyperparameters might be tested. For
+                reproducible results set it to integer.
+
+            early_stopping_rounds (int, optional):
+                Number of rounds with constant performance after which the model fitting stops. This is passed to the
+                fit method of the model for Shapley values estimation, but not for hyperparameter search. Only
+                supported by some models, such as XGBoost and LightGBM.
+
+            eval_metric (str, optional):
+                Metric for scoring fitting rounds and activating early stopping. This is passed to the
+                fit method of the model for Shapley values estimation, but not for hyperparameter search. Only
+                supported by some models, such as [XGBoost](https://xgboost.readthedocs.io/en/latest/parameter.html#learning-task-parameters)
+                 and [LightGBM](https://lightgbm.readthedocs.io/en/latest/Parameters.html#metric-parameters).
+                Note that `eval_metric` is an argument of the model's fit method and it is different from `scoring`.
+        """  # noqa
+        super(EarlyStoppingShapRFECV, self).__init__(
+            clf,
+            step=step,
+            min_features_to_select=min_features_to_select,
+            cv=cv,
+            scoring=scoring,
+            n_jobs=n_jobs,
+            verbose=verbose,
+            random_state=random_state,
+        )
+        if self.search_clf:
+            if self.verbose > 0:
+                warnings.warn(
+                    "Early stopping will be used only during Shapley value estimation step, and not for hyperparameter"
+                    "optimization."
+                )
+
+        if isinstance(early_stopping_rounds, int) and early_stopping_rounds > 0:
+            self.early_stopping_rounds = early_stopping_rounds
+        else:
+            raise (
+                ValueError(
+                    f"The current value of early_stopping_rounds = {early_stopping_rounds} is not allowed. "
+                    f"It needs to be a positive integer."
+                )
+            )
+
+        self.eval_metric = eval_metric
+
+    def _get_feature_shap_values_per_fold(self, X, y, clf, train_index, val_index, sample_weight=None, **shap_kwargs):
+        """
+        This function calculates the shap values on validation set, and Train and Val score.
+
+        Args:
+            X (pd.DataFrame):
+                Dataset used in CV.
+
+            y (pd.Series):
+                Binary labels for X.
+
+            sample_weight (pd.Series, np.ndarray, list, optional):
+                array-like of shape (n_samples,) - only use if the model you're using supports
+                sample weighting (check the corresponding scikit-learn documentation).
+                Array of weights that are assigned to individual samples.
+                Note that they're only used for fitting of  the model, not during evaluation of metrics.
+                If not provided, then each sample is given unit weight.
+
+            clf (binary classifier):
+                Model to be fitted on the train folds.
+
+            train_index (np.array):
+                Positions of train folds samples.
+
+            val_index (np.array):
+                Positions of validation fold samples.
+
+            **shap_kwargs:
+                keyword arguments passed to
+                [shap.Explainer](https://shap.readthedocs.io/en/latest/generated/shap.Explainer.html#shap.Explainer).
+                It also enables `approximate` and `check_additivity` parameters, passed while calculating SHAP values.
+                The `approximate=True` causes less accurate, but faster SHAP values calculation, while
+                `check_additivity=False` disables the additivity check inside SHAP.
+        Returns:
+            (np.array, float, float):
+                Tuple with the results: Shap Values on validation fold, train score, validation score.
+        """
+        X_train, X_val = X.iloc[train_index, :], X.iloc[val_index, :]
+        y_train, y_val = y.iloc[train_index], y.iloc[val_index]
+
+        if sample_weight is not None:
+            clf = clf.fit(
+                X_train,
+                y_train,
+                sample_weight=sample_weight.iloc[train_index],
+                eval_set=[(X_val, y_val)],
+                early_stopping_rounds=self.early_stopping_rounds,
+                eval_metric=self.eval_metric,
+            )
+        else:
+            clf = clf.fit(
+                X_train,
+                y_train,
+                eval_set=[(X_val, y_val)],
+                early_stopping_rounds=self.early_stopping_rounds,
+                eval_metric=self.eval_metric,
+            )
+        # Score the model
+        score_train = self.scorer.scorer(clf, X_train, y_train)
+        score_val = self.scorer.scorer(clf, X_val, y_val)
+
+        # Compute SHAP values
+        shap_values = shap_calc(clf, X_val, verbose=self.verbose, **shap_kwargs)
+        return shap_values, score_train, score_val
